@@ -2301,7 +2301,7 @@ uint8_t
 ipmi_picmg_ipmb_address(struct ipmi_intf *intf) {
 	struct ipmi_rq req;
 	struct ipmi_rs *rsp;
-	char msg_data;
+	uint8_t msg_data;
 
 	if (!intf->picmg_avail) {
 		return 0;
@@ -2336,12 +2336,13 @@ picmg_discover(struct ipmi_intf *intf) {
 	 *  PICMG Extension Version 2.0 (PICMG 3.0 Revision 1.0 ATCA) to
 	 *  PICMG Extension Version 2.3 (PICMG 3.0 Revision 3.0 ATCA)
 	 *  PICMG Extension Version 4.1 (PICMG 3.0 Revision 3.0 AMC)
+	 *  PICMG Extension Version 5.0 (MTCA.0 R1.0)
 	 */
 
 	/* First, check if PICMG extension is available and supported */
 	struct ipmi_rq req;
 	struct ipmi_rs *rsp;
-	char msg_data;
+	uint8_t msg_data;
 	uint8_t picmg_avail = 0;
 
 	memset(&req, 0, sizeof(req));
@@ -2366,8 +2367,9 @@ picmg_discover(struct ipmi_intf *intf) {
 	} else if (rsp->data[0] != 0) {
 	    lprintf(LOG_INFO,"Invalid Get PICMG Properties group extension %#x",
 		    rsp->data[0]);
-	} else if ((rsp->data[1] & 0x0F) != PICMG_ATCA_MAJOR_VERSION
-		&& (rsp->data[1] & 0x0F) != PICMG_AMC_MAJOR_VERSION) {
+	} else if ((rsp->data[1] & 0x0F) != PICMG_EXTENSION_ATCA_MAJOR_VERSION
+		&& (rsp->data[1] & 0x0F) != PICMG_EXTENSION_AMC0_MAJOR_VERSION
+		&& (rsp->data[1] & 0x0F) != PICMG_EXTENSION_UTCA_MAJOR_VERSION) {
 	    lprintf(LOG_INFO,"Unknown PICMG Extension Version %d.%d",
 		    (rsp->data[1] & 0x0F), (rsp->data[1] >> 4));
 	} else {
